@@ -1,5 +1,3 @@
-# vim: tabstop=4 shiftwidth=4 softtabstop=4
-
 # Copyright 2010 OpenStack Foundation
 # All Rights Reserved.
 #
@@ -27,17 +25,6 @@ CLIENT_CLASS = client.Client
 @utils.unauthenticated
 def do_discover(cs, args):
     """Discover Keystone servers, supported API versions and extensions.
-
-    Usage::
-
-        $ keystone discover
-        Keystone found at http://localhost:35357
-
-    - supports version v1.0 (DEPRECATED) here http://localhost:35357/v1.0
-    - supports version v1.1 (CURRENT) here http://localhost:35357/v1.1
-    - supports version v2.0 (CURRENT) here http://localhost:35357/v2.0
-        - and RAX-KSKEY: Rackspace API Key Authentication Admin Extension
-        - and RAX-KSGRP: Rackspace Keystone Group Extensions
     """
     if cs.endpoint:
         versions = cs.discover(cs.endpoint)
